@@ -2,7 +2,7 @@ package Alzabo::ObjectCache::Store::Null;
 
 use vars qw($SELF $VERSION);
 
-$VERSION = sprintf '%2d.%02d', q$Revision: 1.3 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf '%2d.%02d', q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/;
 
 1;
 
@@ -47,10 +47,10 @@ Alzabo::ObjectCache::Store::Null - Doesn't really store anything
 
 =head1 SYNOPSIS
 
-  use Alzabo::ObjectCache( store => 'Alzabo::ObjectCache::Store::Null',
-                           sync  => 'Alzabo::ObjectCache::Sync::BerkeleyDB',
-                           sync_dbm_file => 'somefilename.db',
-                         );
+  use Alzabo::ObjectCache
+      ( store => 'Alzabo::ObjectCache::Store::Null',
+        sync  => 'Alzabo::ObjectCache::Sync::BerkeleyDB',
+        sync_dbm_file => 'somefilename.db' );
 
 =head1 DESCRIPTION
 
