@@ -12,7 +12,7 @@ use Alzabo::Exceptions;
 use Fcntl qw( :flock O_RDONLY O_RDWR O_CREAT );
 use SDBM_File;
 
-$VERSION = sprintf '%2d.%02d', q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf '%2d.%02d', q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/;
 
 1;
 
@@ -83,10 +83,11 @@ Alzabo::ObjectCache::SDBM_File - Uses an SDBM file to sync object caches
 
 =head1 SYNOPSIS
 
-  use Alzabo::ObjectCache( store => 'Alzabo::ObjectCache::Store::Memory',
-                           sync  => 'Alzabo::ObjectCache::Sync::SDBM_File',
-                           sync_dbm_file => 'somefilename.db',
-                           clear_on_startup => 1 );
+  use Alzabo::ObjectCache
+      ( store => 'Alzabo::ObjectCache::Store::Memory',
+        sync  => 'Alzabo::ObjectCache::Sync::SDBM_File',
+        sync_dbm_file => 'somefilename.db',
+        clear_on_startup => 1 );
 
 =head1 DESCRIPTION
 
