@@ -2,12 +2,12 @@ sub mysql_clean_schema
 {
     my %p = @_;
 
-    warn "\n# Removing MySQL database $p{schema_name}\n";
+    print "\n# Removing MySQL database $p{schema_name}\n";
 
     my $s = Alzabo::Create::Schema->new( name => $p{schema_name},
 					 rdbms => 'MySQL' );
 
-    delete $p{rdbms};
+    delete @p{ 'schema_name', 'rdbms' };
     $s->drop(%p);
 }
 
@@ -18,9 +18,9 @@ sub pg_clean_schema
     my $s = Alzabo::Create::Schema->new( name => $p{schema_name},
 					 rdbms => 'PostgreSQL' );
 
-    warn "\n# Removing PostgreSQL database $p{schema_name}\n";
+    print "\n# Removing PostgreSQL database $p{schema_name}\n";
 
-    delete $p{rdbms};
+    delete @p{ 'schema_name', 'rdbms' };
     $s->drop(%p);
 }
 
